#define COMPILATION_OPTIONS "echoping 6.0.2 compiled with cc on i386-1.ports.openbsd.org (i386-unknown-openbsd7.9)\n at 2026-04-24 with options:\n\nHTTP: enabled\nICP: enabled\nOPENSSL: enabled\nGNUTLS: disabled \nSMTP: enabled\nLIBIDN: disabled \nTOS: enabled\nPRIORITY: disabled \n\nPlugins are searched in /usr/local/lib/echoping."

/* $Id$ */
